@@ -16,6 +16,7 @@ from functools import wraps
 from pprint import pprint
 
 import click
+import link_header
 import requests
 from requests import request
 
@@ -145,6 +146,22 @@ def _progress_bar(*args, **kwargs):
     return bar
 
 
+def _recursive_gh_get(href, items):
+    """Recursively get list of GitHub objects.
+
+    See https://developer.github.com/v3/guides/traversing-with-pagination/
+    """
+    response = _request('GET', href)
+    response.raise_for_status()
+    items.extend(response.json())
+    if "link" not in response.headers:
+        return
+    links = link_header.parse(response.headers["link"])
+    rels = {link.rel: link.href for link in links.links}
+    if "next" in rels:
+        _recursive_gh_get(rels["next"], items)
+
+
 @click.group()
 @click.option("--github-token", envvar='GITHUB_TOKEN', default=None,
               help="[default: GITHUB_TOKEN env. variable]")
@@ -227,10 +244,11 @@ def print_release_info(release, title=None, indent=""):
 
 
 def get_releases(repo_name, verbose=False):
-    response = _request(
-        'GET', GITHUB_API + '/repos/{0}/releases'.format(repo_name))
-    response.raise_for_status()
-    releases = response.json()
+
+    releases = []
+    _recursive_gh_get(
+        GITHUB_API + '/repos/{0}/releases'.format(repo_name), releases)
+
     if verbose:
         list(map(print_release_info,
                  sorted(releases, key=lambda r: r['tag_name'])))
@@ -271,26 +289,43 @@ def _update_release_sha(repo_name, tag_name, new_release_sha, dry_run):
     if not refs:
         return
     assert len(refs) == 1
+
+    # If sha associated with "<tag_name>" is up-to-date, we are done.
     previous_release_sha = refs[0]["object"]["sha"]
     if previous_release_sha == new_release_sha:
         return
+
     tmp_tag_name = tag_name + "-tmp"
 
+    # If any, remove leftover temporary tag "<tag_name>-tmp"
+    refs = get_refs(repo_name, tags=True, pattern="refs/tags/%s" % tmp_tag_name)
+    if refs:
+        assert len(refs) == 1
+        time.sleep(0.1)
+        gh_ref_delete(repo_name,
+                      "refs/tags/%s" % tmp_tag_name, dry_run=dry_run)
+
+    # Update "<tag_name>" release by associating it with the "<tag_name>-tmp"
+    # and "<new_release_sha>". It will create the temporary tag.
     time.sleep(0.1)
     patch_release(repo_name, tag_name,
                   tag_name=tmp_tag_name,
                   target_commitish=new_release_sha,
                   dry_run=dry_run)
 
+    # Now "<tag_name>-tmp" references "<new_release_sha>", remove "<tag_name>"
     time.sleep(0.1)
     gh_ref_delete(repo_name, "refs/tags/%s" % tag_name, dry_run=dry_run)
 
+    # Finally, update "<tag_name>-tmp" release by associating it with the
+    # "<tag_name>" and "<new_release_sha>".
     time.sleep(0.1)
     patch_release(repo_name, tmp_tag_name,
                   tag_name=tag_name,
                   target_commitish=new_release_sha,
                   dry_run=dry_run)
 
+    # ... and remove "<tag_name>-tmp"
     time.sleep(0.1)
     gh_ref_delete(repo_name,
                   "refs/tags/%s" % tmp_tag_name, dry_run=dry_run)
@@ -338,6 +373,7 @@ def patch_release(repo_name, current_tag_name, **values):
             headers={'Content-Type': 'application/json'})
         response.raise_for_status()
 
+    # In case a new tag name was provided, remove the old one.
     if current_tag_name != data["tag_name"]:
         gh_ref_delete(
             repo_name, "refs/tags/%s" % current_tag_name,
@@ -348,14 +384,15 @@ def get_assets(repo_name, tag_name, verbose=False):
     release = get_release(repo_name, tag_name)
     if not release:
         raise Exception('Release with tag_name {0} not found'.format(tag_name))
-    response = _request(
-        'GET', GITHUB_API + '/repos/{0}/releases/{1}/assets'.format(
-            repo_name, release["id"]))
-    response.raise_for_status()
-    assets = response.json()
+
+    assets = []
+    _recursive_gh_get(GITHUB_API + '/repos/{0}/releases/{1}/assets'.format(
+        repo_name, release["id"]), assets)
+
     if verbose:
         for i, asset in enumerate(sorted(assets, key=lambda r: r['name'])):
             print_asset_info(i, asset)
+
     return assets
 
 
@@ -435,8 +472,8 @@ def gh_release_create(repo_name, tag_name, asset_pattern=None, name=None,
 @click.option("--target-commitish", default=None)
 @click.option("--name", default=None)
 @click.option("--body", default=None)
-@click.option("--draft", is_flag=True, default=None)
-@click.option("--prerelease", is_flag=True, default=None)
+@click.option("--draft/--publish", is_flag=True, default=None)
+@click.option("--prerelease/--release", is_flag=True, default=None)
 @click.option("--dry-run", is_flag=True, default=False)
 @click.option("--verbose", is_flag=True, default=False)
 @click.pass_obj
@@ -870,17 +907,32 @@ def print_ref_info(ref, indent=""):
 
 
 def get_refs(repo_name, tags=None, pattern=None):
-    response = _request(
-          'GET', GITHUB_API + '/repos/{0}/git/refs'.format(repo_name))
-    response.raise_for_status()
+
+    refs = []
+    _recursive_gh_get(
+        GITHUB_API + '/repos/{0}/git/refs'.format(repo_name), refs)
 
     # If "tags" is True, keep only "refs/tags/*"
-    data = response.json()
+    data = refs
     if tags:
+        tag_names = []
         data = []
-        for ref in response.json():
+        for ref in refs:
             if ref['ref'].startswith("refs/tags"):
                 data.append(ref)
+                tag_names.append(ref["ref"])
+
+        try:
+            tags = []
+            _recursive_gh_get(
+                GITHUB_API + '/repos/{0}/git/refs/tags'.format(repo_name), tags)
+            for ref in tags:
+                if ref["ref"] not in tag_names:
+                    data.append(ref)
+        except requests.exceptions.HTTPError as exc_info:
+            response = exc_info.response
+            if response.status_code != 404:
+                raise
 
     # If "pattern" is not None, select only matching references
     filtered_data = data
@@ -973,6 +1025,25 @@ def gh_ref_delete(repo_name, pattern, keep_pattern=None, tags=False,
     return len(removed_refs) > 0
 
 
+#
+# Commits
+#
+
+def gh_commit_get(repo_name, sha):
+    try:
+        response = _request(
+            'GET',
+            GITHUB_API + '/repos/{0}/git/commits/{1}'.format(repo_name, sha))
+        response.raise_for_status()
+        return response.json()
+    except requests.exceptions.HTTPError as exc_info:
+        response = exc_info.response
+        if response.status_code == 404:
+            return None
+        else:
+            raise
+
+
 #
 # Script entry point
 #
